@@ -5,8 +5,9 @@ import pickle
 
 import numpy as np
 import scipy.io
-from scipy.interpolate import interpn
+from scipy.interpolate import interpn, interp1d
 
+eps = np.finfo(float).eps
 
 class Lookup:
     def __init__(self, filename="MOS.mat"):
@@ -140,11 +141,11 @@ class Lookup:
         except:
             return []
         # appropriate lookup function is called with modefuncmap dict
-        y = self.__modefuncmap.get(mode) (outkeys, varkeys, pars)
+        y = self.__modefuncmap.get(mode) (outkeys, varkeys, vararg, pars)
         
         return y
 
-    def _SimpleLK(self, outkeys, varkeys, pars):
+    def _SimpleLK(self, outkeys, varkeys, vararg, pars):
         """
         Lookup for Modes 1 and 2
 
@@ -177,9 +178,64 @@ class Lookup:
 
         return output
 
-    def _RatioVRatioLK(self):
-        pass
+    def _RatioVRatioLK(self, outkeys, varkeys, vararg, pars):
+        # unpack outkeys and ydata
+        num, den = outkeys
+        ydata =  self.__DATA[num]/self.__DATA[den]
+        # unpack varkeys and xdata
+        num, den = varkeys
+        xdata = self.__DATA[num]/self.__DATA[den]
+        xdesired = vararg
         
+        points = (self.__DATA['L'], self.__DATA['VGS'], self.__DATA['VDS'],\
+            self.__DATA['VSB'])
+        xi_mesh = np.array(np.meshgrid(pars['L'], pars['VGS'], pars['VDS'], pars['VSB'], indexing='ij'))
+        xi = np.rollaxis(xi_mesh, 0, 5)
+        xi = xi.reshape(int(xi_mesh.size/4), 4)
+
+        x = interpn(points, xdata, xi).reshape(len(np.atleast_1d(pars['L'])), \
+            len(np.atleast_1d(pars['VGS'])), len(np.atleast_1d(pars['VDS'])),\
+                 len(np.atleast_1d(pars['VSB'])))
+        
+        y = interpn(points, ydata, xi).reshape(len(np.atleast_1d(pars['L'])), \
+            len(np.atleast_1d(pars['VGS'])), len(np.atleast_1d(pars['VDS'])),\
+                 len(np.atleast_1d(pars['VSB'])))
+        
+        x = np.array(np.squeeze(np.transpose(x, (1, 0, 2, 3))))
+        y = np.array(np.squeeze(np.transpose(y, (1, 0, 2, 3))))
+        
+        if x.ndim == 1:
+            x.shape += (1,)
+            y.shape += (1,)
+
+        dim = x.shape
+        output = np.zeros((dim[1], len(xdesired)))
+        ipkwargs = {'fill_value' : np.nan, 'bounds_error': False}
+        for i in range(0, dim[1]):
+            for j in range(0, len(xdesired)):
+                m = max(x[:, i])
+                idx = np.argmax(x[:, i])
+                if (xdesired[j] > m):
+                    print(f'Look up warning: {num}_{den} input larger than maximum! Output is NaN')
+                if (num.upper() is 'GM') and (den.upper() is 'ID'):
+                    x_right = x[idx:-1, i]
+                    y_right = y[idx:-1, i]
+                    output[i, j] = interp1d(x_right, y_right, **ipkwargs)(xdesired[j])
+                elif (num.upper() is 'GM') and (den.upper() is 'CGG') or (den.upper() is 'CGG'):
+                    x_left = x[1:idx, i]
+                    y_left = y[1:idx, i]
+                    output[i, j] = interp1d(x_left, y_left, **ipkwargs)(xdesired[j])
+                else:
+                    crossings = len(np.argwhere(np.diff(np.sign(x[:,i]-xdesired[j]+eps))))
+                    if crossings > 1:
+                        print('Crossing warning')
+                        return []
+                output[i, j] = interp1d(x[:,i], y[:, i], **ipkwargs)(xdesired[j])
+        
+        output = np.squeeze(output)
+
+        return output
+
     def lookupVGS(self, **kwargs):
         return self.look_upVGS(**kwargs)
 
